jQuery( document).ready( function( $ ){

	"use strict";
	( function(){
		var selector = $('.ipanel-search-input');
		selector.ipanel_search_field({
			fields_container: '#ipanel-fields',
			tabs: '.ipanel-tabs'
		});
		selector.parent().next().on( 'click', function(){
			$(this).prev().find('input').trigger('reset');
		} );
	})();
	$.ipanel = {};
	$.ipanel.addNotice = function( args ){
		var body = $('body'),
			count = body.data( 'ipanel-notices-count' ) == undefined ? 1 : body.data( 'ipanel-notices-count' ),
			top = 0;
		body.find( '> .ipanel-notice:visible' ).each( function(){
			top += $( this ).outerHeight() + 10;
		} );
		body.prepend( '<div class="ipanel-notice" id="ipanel-notice-'+(count + 1)+'" style="display:none;top: '+top+'px"><p>'+args.text+'</p></div>' ).find( '> div#ipanel-notice-' + (count + 1) ).fadeIn();
		setTimeout( function(){
			var _this = body.find( '> div#ipanel-notice-' + (count + 1) );
			if( $.isFunction( args.callback ) && !args.callback.call( _this[0] ) ){
				return;
			}
			_this.fadeOut( function(){
				$(this).remove();
			});
		}, ( args.timeout === undefined ? 3500 : args.timeout ) );
		body.data( 'ipanel-notices-count', count + 1 );
	};
	$.ipanel.block = function(){
		var animation = '<svg width="36" height="36" viewBox="0 0 300 300"\
		xmlns="http://www.w3.org/2000/svg" version="1.1">\
			<path d="M 150,0\
           a 150,150 0 0,1 106.066,256.066\
           l -35.355,-35.355\
           a -100,-100 0 0,0 -70.711,-170.711 z"\
			fill="#FFF">\
				<animateTransform attributeName="transform" attributeType="XML"\
				type="rotate" from="0 150 150" to="360 150 150"\
				begin="0s" dur="1s" fill="freeze" repeatCount="indefinite" />\
			</path>\
			</svg>\
		';
		$.blockUI({
			css: {
				background: 'none',
				border: 0
			},
			message: animation
		});
	};
	$.ipanel.unblock = function(){
		$.unblockUI();
	};
	$.ipanel.showModal = function( args ){
		var
			text = typeof args == 'object' ? args.text : args,
			html = '<div class="ipanel-modal-bg"></div>' +
			'<div class="ipanel-modal"><span class="close">x</span>'+text+'</div>',
			body = $('body');
		body.append( html).find( '.ipanel-modal-bg').fadeIn( 200 );
		body.find('.ipanel-modal').css( "margin-left", function(){
			return - $(this).outerWidth() / 2;
		} ).css( "margin-top", function(){
				return - $(this).outerHeight() / 2;
			}).delay(130).animate( {top: '50%', opacity: 'show'}, 300 );

		body.on( 'click', '.ipanel-modal-bg', function(){
			$(this).next().animate({ top: '-50%', opacity: 'hide' }, 200, function(){
				$(this).prev().fadeOut('fast');
				$(this).add( $(this).prev()).remove();
			});
		});
	};
	$.ipanel.closeModal = function(){
		var body = $('body');
		body.find( '.ipanel-modal' ).animate( { top: '-50%', opacity: 'hide' }, 200, function(){
			$(this).prev().fadeOut('fast');
			$(this).add( $(this).prev() ).remove();
		});
	};
	$(document).on( 'click', '.ipanel-modal .close', function(){
		$.ipanel.closeModal();
	} );
	$.ipanel.ajax = function( request_data, callback ){
		if( typeof request_data !== 'object' || request_data.action === undefined )
			return false;
		var _data = $.extend(
			request_data,
			{
				action: 'ipanel',
				action2: request_data.action,
				panel_id: ipanel.panel_id,
				_nonce: ipanel.nonce
			}
		);
		$.ajax({
			type: 'POST',
			dataType: 'json',
			url: ipanel.ajax_url,
			data:_data,
			success: function(data){
				callback( true, data );
			},
			error: function(MLHttpRequest, textStatus, errorThrown){
				callback( false );
			}
		});

	};

	window.$ = jQuery;

	/*
		Select fields
	*/
	function ipanel_select_fields_init( selector ){
		selector
		.filter( function(){
			return !$(this).hasClass('no-select2');
		})
		.each( function(){
			var options = $(this).data('ipanel-select2');
			if( typeof options != 'object' )
				options = {};
			var o = $(this).select2(options);
			if( $(this).data('sortable') )
				o.siblings().find("ul.select2-choices").sortable({
					containment: 'parent',
					revert: true
				});
		});
	}
	ipanel_select_fields_init( $('select') );



	var id = ipanel.panel_id;

	( function(){

		function change_active_tab( tab ){
			$.cookie('ipanel_'+panel_id+'_current_tab', tab, { expires: 365 });
		}
		function find_tab_by_id( id ){
			return tabs_container.find('a[data-target="'+id+'"]');
		}
		function find_group_by_id( id ){
			return groups_container.find('>div[data-id="'+id+'"]');
		}
		function find_parent( id ){
			return find_tab_by_id( id ).closest('.ipanel-tab-children');
		}
		function display_tab( id ){
			change_active_tab( id );
//			$('body').animate({
//				scrollTop: $('#ipanel-header').offset().top - 40
//			});
			groups_container.find('[data-id="'+id+'"]').animate(
				{
					left: 0,
					opacity: 'show'
				},
				150
			);
		}
		function hide_tab( id, callback ){
			return find_group_by_id( id).animate(
				{
					left: -30,
					opacity: 'hide'
				},
				150,
				function(){
					if( $.isFunction( callback ) )
						callback.apply( this );
				}
			);
		}
		function has_child( id ){
			return find_tab_by_id( id ).next().is('.ipanel-tab-children');
		}
		function has_field( tab_id ){
			return find_group_by_id( tab_id).find('>*').size() > 0;
		}
		function is_child( id ){
			return find_tab_by_id( id ).parent().parent().is('.ipanel-tab-children');
		}
		function show( current_tab ){
			if( is_child( current_tab ) ){

				find_parent( current_tab ).slideDown();
				find_parent( current_tab ).prev().addClass('active');

			} else {
				tabs_container.find('ul.ipanel-tab-children').slideUp();
				tabs_container.find('.active').removeClass('active');

				var target = find_tab_by_id( current_tab );
				target.next('ul').slideDown();
				target.addClass('active');
				if( !has_field( current_tab ) && has_child( current_tab ) ){
					var group = target.next().find('a:first').data('target');
					return display_tab( group );
				}

			}

			display_tab( current_tab );

		}

		var
			panel_id = ipanel.panel_id,
			groups_container = $('#ipanel-fields'),
			tabs_container = $('.ipanel-tabs');
		( function(){
			var current_tab = $.cookie('ipanel_'+panel_id+'_current_tab');
			if( !current_tab || find_tab_by_id( current_tab ).size() === 0 )
				current_tab = tabs_container.find('a:first').data('target');

			show( current_tab );



		})();

		tabs_container.find('a').click( function( e ){
			e.preventDefault();

			var
				current_tab = groups_container.find( '> div:visible').data('id'),
				this_id = $(this).data('target');
			hide_tab( current_tab, function(){
				show( this_id );
			} );
		});
	} )();



//	$.ipanel.change_active_tab = function( panel_id, tab ){
//		$.cookie('ipanel_'+panel_id+'_current_tab', tab, { expires: 365 });
//	};
//
//	$.ipanel.get_panel_active_tab = function( id ){
//		return $.cookie('ipanel_'+id+'_current_tab') === undefined ? false : $.cookie('ipanel_'+id+'_current_tab');
//	};
//
//	$.ipanel.display_tab = function( id ){
//		var
//			selector = ".ipanel-group[data-id='"+id+"']",
//			container = $('#ipanel-fields'),
//			target,
//			has_child = $('#ipanel-tabs').find( '[data-target="'+id+'"]').next().is('ul');
//		target = container.find(selector);
//
//		if( target.find('>*').size() === 0 && has_child )
//			log( 1 );
////			target;
//		if( container.find(selector).length !== 1 ){
//			return;
//		}
//		$.ipanel.change_active_tab( ipanel.panel_id, id );
//		if( container.find('.ipanel-group:visible').length ){
//			container.find('.ipanel-group:visible').animate( { left: '-30px', opacity: 'hide' }, 140, function(){
//				target.animate({
//					left: '0',
//					opacity: 'show'
//				}, 140 );
//			} );
//		}
//		else {
//			target.css('left',0).fadeIn();
//		}
//		$('.ipanel-tabs').find('a').removeClass('active').end().find('a[data-target="'+id+'"]').addClass('active');
//	};
//
//	$.ipanel.init = function(){
//		var current_tab = $.ipanel.get_panel_active_tab( ipanel.panel_id );
//		if( current_tab ){
//			$.ipanel.display_tab( current_tab );
//		} else {
//			var id = $('.ipanel-group[data-id]:first').data('id');
//			$.ipanel.display_tab(id);
//		}
//		$('.ipanel-tabs').find('a').click( function(){
//			var _this = $(this),
//				id = _this.data('target'),
//				is_child = _this.closest('ul').is('.ipanel-tab-children'),
//				child = _this.siblings('ul.ipanel-tab-children'),
//				has_child = child.size() > 0,
//				target_group = $('#ipanel-fields').find('>div.ipanel-group[data-id="'+id+'"]');
//
//			if( has_child ){
//				if( !child.is(':visible') ){
//					$('.ipanel-tabs ul.ipanel-tab-children').not( child ).slideUp();
//					child.slideDown();
//				}
//			}
//
//			if( target_group.size() !== 1 || $.ipanel.get_panel_active_tab( ipanel.panel_id ) == id )
//				return;
//
//			$.ipanel.display_tab( id );
//
//		} );
//
//	}();




	$(document).on( 'click', '.ipanel-reset', function( e ){
		e.preventDefault();
	});

	$('#ipanel-fields').on( 'keypress', ':input', function( e ){
		var keycode = (event.keyCode ? event.keyCode : event.which);
		if( keycode == 13 && !$(e.target).is('textarea') )
			$('#ipanel-form').trigger('submit');
	} );
	$(document).on( 'submit', '#ipanel-form', function( e ){
		e.preventDefault();
		$(this).ajaxSubmit({
			dataType: 'json',
			beforeSubmit: function(){
				$(document).trigger( "ipanelBeforePanelSave", [ ipanel.panel_id ] );
			},
			success: function( data ){
				$(document).trigger( "ipanelPanelSave", [ ipanel.panel_id ] );
				$(document).trigger( "ipanelPanelSaveCallback", [ ipanel.panel_id ] );
				$.ipanel.addNotice({
					text: data.msg
				});
			},
			error: function(){
				$.ipanel.addNotice({
					text: ipanel.messages.error_try_again
				});
				$(document).trigger( "ipanelPanelSaveCallback", [ ipanel.panel_id ] );
				$(document).trigger( "ipanelPanelSaveError", [ ipanel.panel_id ] );
			}
		})
	});

	
	$(document).on( 'click', '.ipanel-repeater-item .repeater-item-header', function( e ){
		var body = $(this).next();
		if( body.is(':visible') )
			body.slideUp('fast');
		else
			body.slideDown( 'fast' );
	});
	$( ".ipanel-section.repeater").find('.ipanel-controls').sortable({
		handle: '[data-id="sortable-handle"]',
		revert: true
	});

	$.ipanel.duplicate_repeater = function(){
		var html = $( this ).clone(),
			count = $(this).parent().find('>*').length + 1,
			bottom_margin = $(this).css('margin-bottom'),
			regex = new RegExp( /^data\[.+]\[\d+\]/ );
//		$.each( html.find( '[name^=data]' ), function(){
//			$(this).attr( 'name', function( index, name ){
//				$(this).attr( 'name', name.replace( /\[\d+\]/g, '['+count+']' ) );
//			} );
//		} );
		$.each( html.find( '*' ), function(){
			var _this = $(this);
			$.each( $(this).attrs(), function( i, o ){
				if( regex.test( o ) ){
					_this.attr(
						i,
						function( index, attr ){
							return attr.replace( /\[\d+\]/, '['+count+']' );
						}
					);
				}
			} );
		} );

		
		html
			.find('.select2-container').remove()
			.end()
			.find('select').removeAttr('class');
		ipanel_select_fields_init( html.find('select') );


		html.find('.ipanel-date-picker').removeClass('hasDatepicker').removeAttr('id');
		ipanel_date_picker_init( html.find('.ipanel-date-picker') );


		html.find('.ipanel-slider').removeAttr( 'class').addClass('ipanel-slider').removeAttr('aria-disabled').children().remove();
		ipanel_slider_init( html.find('.ipanel-slider') );


		$.when( html.find( '.ipanel-code' ).removeAttr('id').removeAttr( 'class' ).addClass( 'ipanel-code').find( '>*').remove() ).then( function(){
			ipanel_code_field_init( html.find('.ipanel-code') );
		} );

		ipanel_color_picker_init( html.find('input[type=text].ipanel-color') );

		ipanel_ajax_uploader_init( html.find('.ipanel-ajax-uploader-file-input') );

		if( html.find('.ipanel-wpcolor').size() ){
			var wp_color = html.find('.ipanel-wpcolor'),
				insert_place = wp_color.closest( '.ipanel-controls'),
				wp_color_input = wp_color[0].outerHTML;
			wp_color.closest( '.wp-picker-container').remove();
			insert_place.append( wp_color_input );
			insert_place.find('input[type=text]').wpColorPicker();
		}



		html.hide().css({ 'opacity':0, margin: 0 }).insertAfter( this ).animate( { height: 'show', marginBottom: bottom_margin }, function(){ $(this).delay(100).animate({ opacity: 1 }); } ); //fadeOutUpBig
	};
	
	$( document ).on( 'click', '.ipanel-repeater-control', function( e ){
		switch( $(this).data('id') ){
			case( 'duplicate-repeater' ):
				$.ipanel.duplicate_repeater.apply( $(this).closest('.ipanel-repeater-item')[0] );
				break;
			case( 'remove-item' ):
				var section = $(this).closest('.ipanel-repeater-item');
				if( $(this).closest('.ipanel-controls').find('>*').size() == 1 )
					return false;
				if( confirm( ipanel.messages.confirm ) )
					section.animate(
						{ opacity: 0 },
						function(){
							$(this).animate({ height: 'hide', marginBottom: 0 }, function(){
								$(this).remove();
							});
						});

				break;
		}
		return false;
	} );
	$('.close-repeater-body, .open-repeater-body').on( 'click', function(){
		if( $(this).is( '.open-repeater-body' ) ){
			$(this).closest( '.ipanel-section' ).find('.repeater-body').slideDown();
		} else if( $(this).is( '.close-repeater-body' ) ){
			$(this).closest( '.ipanel-section' ).find('.repeater-body').slideUp();
		}
	} );


	/* Color Fields */
	function ipanel_color_picker_init( element ){
		element.each( function( i, o ){
			var _this = $(this);
			var init = _this.ColorPicker({
				color: $(this).val(),
				onShow: function (colpkr) {
					$(colpkr).fadeIn('fast');
					return false;
				},
				onHide: function (colpkr) {
					$(colpkr).fadeOut('fast');
					return false;
				},
				onChange: function (hsb, hex, rgb) {
					_this.val( "#" + hex );
					_this.prev().css({ 'background-color' : "#"+hex });
				}
			});
			_this.data( '_color_object_', init );
		} );

	}
	ipanel_color_picker_init( $('input[type=text].ipanel-color') );



	/* Wordpress Color picker */
	$('input[type=text].ipanel-wpcolor').wpColorPicker();


	$(document).on( 'click', '.ipanel-image-multicheckbox-option img', function( event ){
		var parent  = $(this).parent(),
			checked = parent.hasClass('checked');
		if( checked ){
			parent.removeClass('checked').addClass('unchecked');
		} else {
			parent.removeClass('unchecked').addClass('checked');
		}
	} );

	$(document).on( 'click', '.ipanel-image-radio-option img', function( event ){
		var parent  = $(this).parent();
		parent.removeClass('checkec unchecked').siblings().removeClass('checked').addClass('unchecked').end().addClass('checked');
	} );


	$(document).on( 'click', '.ipanel-heading > h3', function( event ){
		var input = $(this).parent().siblings('.ipanel-controls').find(':input:first');
		input.focus();
	});


	$('.ipanel-media-items-container[data-sortable]').sortable();
	$(document).on( 'click', '.ipanel-open-media', function( event ){
		event.preventDefault();
		var _this = $(this), container = _this.siblings('.ipanel-media-items-container'), multiple = $(this).data('multiple'), options = {};
		options.multiple = multiple;
		if( _this.data('frame-title') )
			options.title = _this.data('frame-title');
		options.button = {};
		if( _this.data('frame-button-text') )
			options.button.text = _this.data('frame-button-text');
		var media = wp.media(options)
		.on('select', function() {
			$.each( media.state().get('selection').toJSON(), function( i, o ){
				var url = o.url, id = o.id, name = _this.data('name');
				if( container.find( '> [data-url="'+url+'"]').size() === 0 ){
					var html = container.next().next().html();
					container.append( html.replace( /{url}/g, url ).replace( '{name}', name ) );
				}
			} );
		})
		.open();
	});


	$(document).on( 'click', '.ipanel-media-items-container span.remove, .ipanel-ajax-uploader-container span.remove', function(){
		if( confirm( ipanel.messages.confirm ) )
			$(this).parent('div').remove();
	} );


	$('.ipanel-ajax-uploader-container[data-sortable]').sortable();
	function ipanel_ajax_uploader_init( element ){
		element.each( function(){
			var _this = $(this), container = _this.parent().siblings('.ipanel-ajax-uploader-container');
			_this.fileupload({
				singleFileUploads: false,
					formData: {
					action: 'ipanel',
					action2: 'upload_file',
					panel_id: ipanel.panel_id,
					_nonce: ipanel.nonce,
					field_name: _this.attr('name').replace( /\[\]$/, '' )
				},
				url: ipanel.ajax_url,
				dataType: 'json',
				start: function (e) {
					$(e.target).next().fadeIn();
				},
				done: function (e, data) {
					$(e.target).next().animate( { width: 0 }, 50, function(){ $(this).hide() });
					var d = data.result, warp = _this.closest('.ipanel-ajax-uploader-wrapper'), count;
					if( !d.status )
						return;
					$.each( d.data, function( i, o ){
						if( o.status ){
							count = parseInt( container.data('__items_count__') ) + 1;
							container.data( '__items_count__', count );
							container.append( '<div>' +
								'<input type="text" name="'+_this.data('url-name').replace( '[__index__]', '['+count+']' )+'" value="'+ o.url+'" readonly/>' +
								'<input type="hidden" name="'+_this.data('file-name').replace( '[__index__]', '['+count+']' )+'" value="'+ o.file+'" readonly/>' +
								'<span class="remove">x</span></div>' );
						}
						setTimeout( 
							function(){
								$.ipanel.addNotice({
									text : o.msg,
									timeout : o.msg.length * 35
								});
							},
							i * 50
						);
					});
				},
				progressall: function (e, data) {
					var progress = parseInt( data.loaded / data.total * 100, 10);
					$(e.target).next().dequeue().animate({
						width: progress + '%'
					});

				},
				dropZone: _this.closest('.ipanel-section')
			});
		});
	}
	ipanel_ajax_uploader_init($('.ipanel-ajax-uploader-file-input'));



	$( document).on( 'click', '.ipanel-reset-panel', function(e){
		e.preventDefault();
		if( confirm( ipanel.messages.confirm ) ){
			$(document).trigger( 'ipanelBeforePanelReset' );
			$.ipanel.ajax( { action: 'reset_panel' }, function( status, data ){
				$.ipanel.addNotice({
					text: data.msg
				});
				if( status ){
					setTimeout( function(){
						location.reload();
					}, data.reload_time);
					$(document).trigger( 'ipanelPanelReset' );
				}
			} );
		}
	} );


	// Field description
	$(document).on( 'click', '.ipanel-nonrepeater-section > .ipanel-desc > i', function(){
		var target = $(this).next();
		if( target.is(':visible') ){
			target.fadeOut('fast');
		}
		else {
			target.fadeIn('fast').css( {'margin-bottom': -target.height()/2} );
		}
	} );


	// Export Fields
	$( document ).on( 'ipanelPanelSave', function(){
		ipanel.is_saved = 1;
	});
	$(document).on( 'click', '.ipanel-export-control', function( e ){
		e.preventDefault();
		var _this = $(this);
		if( !_this.data('id') )
			return;
		switch( _this.data('id') ){
			case( 'download' ):
				var redirect_params = [
					_this.attr('href'),
					{
						action: 'ipanel',
						action2: 'download_export',
						panel_id: ipanel.panel_id,
						_nonce: ipanel.nonce
					},
					'post'
				];
				if( ipanel.is_saved == 0 ){
					if( confirm( "It seems you've not saved the panel yet. Do you want save it first?" ) ){
						$('.ipanel-save').trigger( 'click' );
						$( document ).on( 'ipanelPanelSaveCallback', function(){
							$().redirect.apply( window, redirect_params );
						});
					}
				}
				else {
					$().redirect.apply( window, redirect_params );
				}
					
				break;
			case( 'get-code' ):
				$(document).trigger( 'ipanelBeforeGetExportCode' );
				$.ipanel.ajax( 
					{
						action: 'get_export_code'
					},
					function( status, data ){
						$(document).trigger( 'ipanelAfterGetExportCode' );
						if( !status ){
							$.ipanel.addNotice({
								text: ipanel.messages.error_try_again
							});
							return;
						}
						var html = $( _this.siblings('script').html() );
						_this.siblings('div').remove().end().after( html.find('textarea').val( data.data ).end() );
					}
				);
				break;

		};
		
	});
	
	
	// Export Fields

	$('.ipanel-import-file').fileupload({
		singleFileUploads: true,
		formData: {
			action: 'ipanel',
			action2: 'import_panel',
			'import_type': 'file',
			panel_id: ipanel.panel_id,
			_nonce: ipanel.nonce
		},
		url: ipanel.ajax_url,
		dataType: 'json',
		start: function (e) {
			$(document).trigger('ipanelBeforeImport');
		},
		done: function (e, data) {
			$(document).trigger('ipanelAfterImport', [data.result.status]);
			if( data.result.status )
				setTimeout( function(){
					location.reload();
				}, data.result.redirect_after );
			$.ipanel.addNotice({
				text: data.result.msg
			});
		},
		progressall: function (e, data) {
			var progress = parseInt( data.loaded / data.total * 100, 10);
		}
	});
	$('.ipanel-import-control[data-id="code"]').on( 'click', function(){
		$.ipanel.showModal( $(this).next().html() );
	} );
	$(document).on( 'click', '#ipanel-import-code-submit', function(e){
		e.preventDefault();
		var
			_parent = $(this).parent(),
			data = _parent.prev().find('textarea').val(),
			regex = new RegExp("^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{4})$");
		_parent.siblings('.msg').fadeOut();

		if( !data ){
			_parent.siblings('.empty-field').fadeIn();
			return false;
		}
		if( !regex.test( data ) ){
			_parent.siblings('.invalid').fadeIn();
			return false;
		}
		$(document).trigger('ipanelBeforeImport');
		$.ipanel.ajax(
			{
				action: 'import_panel',
				import_type : 'code',
				code: data
			},
			function( s, res ){
				$(document).trigger('ipanelAfterImport', [ s ]);
				$.ipanel.closeModal();
				if( s )
					setTimeout( "location.reload();", res.redirect_after );
				$.ipanel.addNotice({
					text: res.msg
				});
			}
		);
	} );

	// Code fields
	function ipanel_code_field_init( element ){
		element.each( function(){
			var _this = $(this);
			var editor = ace.edit( this ), options = $(this).data('options');
			if( typeof options != 'object' )
				options = $.parseJSON( options );
			editor.setValue( _this.prev().val() );
			editor.setShowPrintMargin(false);
			editor.setTheme("ace/theme/"+options.theme);
			editor.getSession().setMode("ace/mode/"+options.lang);
			editor.on( 'change', function( o ){
				_this.prev().val( editor.getValue() );
			});
			_this.data('_ace_editor_', editor)
		});
	}
	ipanel_code_field_init( $('.ipanel-code') );
	

	function ipanel_slider_init( element ){
		element.each( function(){
			var options = $(this).data('options'), has_tooltip = false;
			if( typeof options != 'object' )
				options = {};
			if( options['has-tooltip'] )
				has_tooltip = true;
			options.slide = function( event, ui ){
				$(this).next().val( ui.value );
				$(this).find('.value').text( ui.value );
			};
			options.create = function( event, ui ){
				$(this).find('.ui-slider-handle').append( $(this).prev().html() );
			};
			options.value = $(this).next().val();
			$(this).slider(options);
		} );
	}
	ipanel_slider_init( $('.ipanel-slider') );

	function ipanel_date_picker_init( element ){
		element.each( function(){
			var options = $(this).data('options');
			if( typeof options != 'object' )
				options = {};
			$(this).datepicker(options);
		} );
	}
	ipanel_date_picker_init( $('.ipanel-date-picker') );


	$(document).on( 'click', '.ipanel-section-container > h3', function(){
		var body = $(this).next();
		if( body.is(':visible') )
			body.slideUp();
		else
			body.slideDown();
	} );


	/*
	 Conditional Logics
	 */
	function ipanel_conditional_logics_init( element ){

		function callback( val, data ){
			function _do(){
				$.each( data.result, function( event, target ){
					switch( event ){
						case( 'show' ):
							$('#ipanel-fields').find( '> div > div[data-id="'+target+'"]').show();
							break;
						case( 'hide' ):
							$('#ipanel-fields').find( '> div > div[data-id="'+target+'"]').hide();
							break;
					}
				} );
			}
			if( typeof data.result != 'object' )
				return false;
			var
				operators = [ '==', '===', '!=', '!==' ],
				operator = data.operator && $.inArray( data.operator, operators ) ? data.operator : '==';
			switch ( operator ){
				case( '==' ):
				default:
					if( val == data.value ){
						_do();
					}
					break;
				case( '===' ):
					if( val === data.value ){
						_do();
					}
					break;
				case( '!=' ):
					if( val != data.value ){
						_do();
					}
					break;
				case( '!==' ):
					if( val !== data.value ){
						_do();
					}
					break;
			}

		}

		element.each( function(){
			var _this = $(this), type = _this.data('type');
			$.each( _this.data('conditional-logics'), function( i, data ){
				switch ( type ){
					case( 'text' ):
					case( 'textarea' ):
						_this.find( ':input:first' ).on( 'keyup', function(){
							callback( $(this).val(), data );
						} );
						break;
					case( 'code' ):
						var input = _this.find('.ipanel-code'), editor = input.data('_ace_editor_');
						editor.on( 'change', function( ){
							callback( editor.getValue(), data );
						} );
						break;
					case( 'checkbox' ):
						_this.find( ':checkbox:first').change( function(){
							callback( $(this).is(':checked') ? 1 : 0, data );
						} );
						break;
					case( 'select' ):
						_this.find( 'select:first').change( function(){
							callback( $(this).val(), data );
						} );
						break;
					case( 'radio' ):
					case( 'image_radio' ):
						_this.find( ':radio').change( function(){
							callback( _this.find( ':radio:checked').val(), data );
						} );
						break;
					case( 'wp_editor' ):
						window.onload = function(){
							var _id = _this.find('.ipanel-wp-editor-container:first').data('field-name'), tinymce_editor = tinyMCE.get( _id );
							if( tinymce_editor ){
								tinymce_editor.onKeyUp.add( function( ed ) {
									var content = ed.getContent();
									callback( content, data );
								});
							}
						};
							break;

				}
			} );
		} );
	}
	ipanel_conditional_logics_init( $('.has-conditional-logic') );


	/*
		Sorter Filed Type
	 */
	function ipanel_sortable_init( element ){
		function make_equal(){
			element.find('.items-container').removeAttr('style');
			function find_the_tallest(){
				var maxHeight = -1;
				element.find('.items-container').each(function() {
					if( maxHeight > $(this).actual('height') ){
					} else {
						maxHeight = $(this).actual('height');
					}
				});
				return maxHeight;
			}
			element.find('.items-container').each( function(){
				$(this).height(find_the_tallest());
			} );
			log( find_the_tallest() );
		}
		element.each( function(){
			$(this).find('.items-container').sortable({
				connectWith: $(this).find('.items-container'),
				placeholder: "ipanel-ui-state-highlight",
				over: function(){
					make_equal();

				},
				create: function(){
					make_equal();
				},
				start: function(e, ui ){
					ui.placeholder.height(ui.helper.outerHeight());
				},
				update: function( event, ui ){
					make_equal();
					ui.item.find('input').attr('name', function( index, name ){
						return name.replace( $( ui.sender).parent().data('id'), ui.item.parent().parent().data('id') );
					});
				}
			});
		} );
	}
	ipanel_sortable_init( $('.ipanel-sortable-container') );

});

