<?php

	if( !class_exists( 'iPanel_Fields' ) ){
		require_once 'ipanel-fields.php';
	}

	if( !class_exists( 'iPanel_Template' ) ){

		class iPanel_Template extends iPanel_Fields{

			protected $_id, $_panel, $template;


			/*
			 * Constructor function
			 *
			 * @param (string) panel ID
			 * @param (array)  panel data
			 * @param (array)  values array
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __construct( $panel_id, &$panel, &$values ) {
				$this->_supported_fields[] = 'export';
				$this->_supported_fields[] = 'import';
				$this->_id 	  			 		  =  $panel_id;
				$this->_panel			 		  =& $panel;
				$this->template 		 		  =  array();
				$this->template['name']  		  =  $panel['settings']['name'];
				$this->template['id']	 		  =  $panel_id;
				$this->template['fields'] 		  =  $this->generate_fields( $this->_panel['fields'], 'section', $values );
				$this->template['url'] 		 	  =  $this->_panel['settings']['url'];
				$this->template['hidden_fields']  =  $this->_template_hidden_fields();
				$this->template['tabs']		 	  =  $this->generate_tabs_html();
			}

			/*
			 * Export field type
			 *
			 * @param (array) Field array
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __export__( $field ) {
				$url = admin_url('admin-ajax.php');
				ob_start();
				?>
				<div class="ipanel-export">
					<a href="<?php echo $url; ?>" class="ipanel-button-dark ipanel-export-control" data-id="download">Download</a>
					<a href="#" class="ipanel-button-dark ipanel-export-control" data-id="get-code">Get code</a>
					<script type="text/html">
						<div class="ipanel-export-code">
							<textarea readonly></textarea>
						</div>
					</script>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Import field type
			 *
			 * @param (array) Field array
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __import__( $field ) {
				ob_start();
				?>
				<div class="ipanel-import">
					<a href="#" class="ipanel-button-dark ipanel-import-control" data-id="code">Import Code</a>
					<script type="text/html">
						<div id="ipanel-import-code-container" style="text-align: center">
							<p>
								<label>
									<?php echo __( 'Enter code', $this->_panel['settings']['text_domain'] ); ?>
									<br/>
									<textarea></textarea>
								</label>
							</p>
							<p><a href="#" id="ipanel-import-code-submit" class="button button-primary"><?php echo __( 'Import', $this->_panel['settings']['text_domain'] ); ?></a></p>
							<p class="msg empty-field"><?php echo __( 'The import field is empty.', $this->_panel['settings']['text_domain'] ); ?></p>
							<p class="msg invalid"><?php echo __( "It seems the text you've entered is not valid!", $this->_panel['settings']['text_domain'] ); ?></p>
						</div>
					</script>
					<label class="ipanel-button-dark ipanel-import-control" data-id="file" style="margin-top: -3px;">
						<span>Import File</span>
						<input type="file" class="ipanel-import-file" style="display: none" name="file" />
					</label>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Build field name
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function field_name( &$field, $before = 'data[', $after = ']' ) {
				$name = '';
				if( isset( $field['repeater_item'] ) && $field['repeater_item'] == true && isset( $field['repeater_pos'] ) ) {
					$_filed = $field;
					unset( $_filed['repeater_item'], $_filed['repeater_pos'] );
					$_name = $this->field_name($_filed, '[', ']' );
					$name .= "{$before}{$field['repeater_id']}{$after}[{$field['repeater_pos']}]{$_name}";
				}
				else {
					if( $field['type'] == 'multicheckbox' || $field['type'] == 'multiselect' || $field['type'] == 'image_checkbox' || $field['type'] == 'media' ){
						$name .= "{$before}{$field['id']}{$after}[]";
					} else if( $field['type'] == 'ajax_uploader' ) {
						$input_type = $field['__type__'];
						$name .= "{$before}{$field['id']}{$after}[__index__][{$input_type}]";
					} else if( $field['type'] == 'typography' ) {
						$input_type = $field['__type__'];
						$name .= "{$before}{$field['id']}{$after}[{$input_type}]";
					} else if( $field['type'] == 'sorter' ) {
						$name .= "{$before}{$field['id']}{$after}[{$field['__group__']}][{$field['__item__']}]";
					} else {
						$name .= "{$before}{$field['id']}{$after}";
					}

				}
				return apply_filters( 'ipanel_field_name', $name, $field );
			}

			/*
			 * Generate tabs array
			 *
			 *
			 *
			 * @param (type) about this param
			 *
			 * @since 2.0
			 * @access public
			 * @return array
			 */
			public function generate_tabs_array() {
				$fields =& $this->_panel['fields'];

				// Generate Tabs Array
				$tabs_array		 = array();
				$prev_tab_key	 = 0;
				$menu_items      = array();

				foreach ( (array) $fields as $field )
					if ( $field['type'] == 'tab' || $field['type'] == 'subtab' )
						$tabs_array[] = $field;

				foreach ( $tabs_array as $k => $v ) {
					$token = $v['id'];
					$v['token'] = $token;
					if ( $v['type'] == 'tab' ) {
						$menu_items[$token] = $v;
						$prev_tab_key 		= $token;
					}
					if ( $v['type'] == 'subtab' ) {
						$menu_items[$prev_tab_key]['children'][] = $v;
					}
				}

				return $menu_items;

			}


			/*
			 * Hidden fields for template
			 *
			 * @since 2.0
			 * @access protected
			 * @return string
			 */
			protected function _template_hidden_fields() {
				ob_start();
				?>
				<input type="hidden" name="action" value="ipanel"/>
				<input type="hidden" name="action2" value="save_panel"/>
				<input type="hidden" name="_nonce" value="<?php echo wp_create_nonce( 'ipanel' ); ?>"/>
				<input type="hidden" name="panel_id" value="<?php echo $this->_id; ?>"/>
				<?php
				return ob_get_clean();
			}
			


			/*
			 * Default admin template
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function default_admin_theme( $template ) {
				require_once dirname(__FILE__) . "/../templates/default-admin-theme/template.php";
			}




			/*
			 * Generate tabs HTML
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function generate_tabs_html( $tabs = false, $container_class = 'ipanel-tabs' ) {


				if( $tabs == false )
					$tabs = $this->generate_tabs_array();

//				_print_r( $tabs );

				if( empty( $tabs ) || !is_array( $tabs ) )
					return '';

				$output = '<ul class="'.$container_class.'">';

				foreach( (array) $tabs as $tab ){
					if( empty( $tab['name'] ) || !is_string( $tab['name'] ) || empty( $tab['id'] ) || !is_string( $tab['id'] ) )
						continue;
					$has_child = false;
					if( ( !empty( $tab['children'] ) && is_array( $tab['children'] ) ) || ( !empty( $tab['childrens'] ) && is_array( $tab['childrens'] ) ) ){
						$has_child = true;
						$childrens = ( !empty( $tab['childrens'] ) && is_array( $tab['childrens'] ) )
							? $tab['childrens']
							: $tab['children'];
					}
					$output .= '<li class="'.( $has_child ? 'has-child' : '' ).'">';


					$output .= '<a data-target="'.$tab['id'].'">'.$tab['name'].( $has_child ? '<i></i>' : '' ).'</a>';
					if( $has_child ){
						$output .= call_user_func( array( $this, 'generate_tabs_html' ), $childrens, 'ipanel-tab-children' );
					}
					$output .= '</li>';
				}

				$output .= '</ul>';
				return $output;

			}

			/*
			 * Display the output
			 *
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function display() {
				ob_start();
				do_action( 'ipanel_before_callback_wrapper' );
				do_action( 'ipanel_before_'.$this->_id.'_callback_wrapper' );
				$callback = ( isset( $this->_panel['settings']['template'] ) && is_callable( $this->_panel['settings']['template'] ) )
					? $this->_panel['settings']['template']
					: array( $this, 'default_admin_theme' );
				call_user_func( $callback, $this->template );
				do_action( 'ipanel_after_callback_wrapper' );
				do_action( 'ipanel_after_'.$this->_id.'_callback_wrapper' );
				return ob_get_clean();
			}

		}

	}