<?php

	if( !function_exists( '_print_r' ) ){
		function _print_r( $data ){
			echo '<pre>' . print_r( $data, true ) . '</pre>';
		}
	}

	if( !function_exists( 'ipanel_get_panel' ) ){
		function ipanel_get_panel( $panel_id ){
			$options = apply_filters( 'ipanel_options', array() );
			if(
				isset( $options[ $panel_id ] )
				&& is_array( $options[ $panel_id ] )
				&& !empty( $options[ $panel_id ]['settings'] )
				&& is_array( $options[ $panel_id ]['settings'] )
				&& !empty( $options[ $panel_id ]['fields'] )
				&& is_array( $options[ $panel_id ]['fields'] )
			)
				return $options[ $panel_id ];
			return false;
		}
	}

	if( !function_exists( 'ipanel_find_field' ) ){
		function ipanel_find_field( $panel_id, $id, $ignore_tabs = false ){
			$options = ipanel_get_panel_fields( $panel_id );
			if( $options === false )
				return false;
			foreach( (array) $options as $field ){
				if( $field['type'] == 'tab' && $ignore_tabs )
					continue;
				if( isset( $field['id'] ) && $field['id'] == $id )
					return $field;
			}
			return false;
		}
	}

	if( !function_exists( 'ipanel_get_panel_settings' ) ){
		function ipanel_get_panel_settings( $panel_id ){
			$panel = ipanel_get_panel( $panel_id );
			return $panel === false ? false : $panel['settings'];
		}
	}

	if( !function_exists( 'ipanel_get_panel_option_name' ) ){
		function ipanel_get_panel_option_name( $panel_id, $default_format = 'ipanel_%s' ){
			$settings = ipanel_get_panel_settings( $panel_id );
			if( !empty( $settings['option_name'] ) && is_string( $settings['option_name'] ) ){
				return $settings['option_name'];
			} else {
				return sprintf( $default_format, $panel_id );
			}
		}
	}

	if( !function_exists('hex2rgb') ){
		function hex2rgb($hex) {
			$hex = str_replace("#", "", $hex);

			if(strlen($hex) == 3) {
				$r = hexdec(substr($hex,0,1).substr($hex,0,1));
				$g = hexdec(substr($hex,1,1).substr($hex,1,1));
				$b = hexdec(substr($hex,2,1).substr($hex,2,1));
			} else {
				$r = hexdec(substr($hex,0,2));
				$g = hexdec(substr($hex,2,2));
				$b = hexdec(substr($hex,4,2));
			}
			$rgb = array($r, $g, $b);
			//return implode(",", $rgb); // returns the rgb values separated by commas
			return $rgb; // returns an array with the rgb values
		}
	}

	if( !function_exists( 'is_assoc' ) ){
		function is_assoc( $array ) {
			return (bool)count(array_filter(array_keys($array), 'is_string'));
		}
	}


	if( !function_exists( 'ipanel_get_panel_fields' ) ){
		function ipanel_get_panel_fields( $panel_id ){
			$panel = ipanel_get_panel_fields( $panel_id );
			return $panel === false ? false : $panel['fields'];
		}
	}


	if( !function_exists( 'ipanel_restore_panel_default_values' ) ){
		function ipanel_restore_panel_default_values( $panel_id ){
			$panel = ipanel_get_panel( $panel_id );
			if( $panel === false )
				return false;
			$values = array();
			$option = !empty( $panel['settings']['option_name'] ) && is_string( $panel['settings']['option_name'] ) ? $panel['settings']['option_name'] : 'ipanel_' . $panel_id;
			foreach( (array) $panel['fields'] as $field ){
				if( $field['type'] == 'sorter' )
					continue;
				if( !empty( $field['std'] ) && !empty( $field['id'] ) && is_string( $field['id'] ) )
					$values[ $field['id'] ] = $field['std'];
			}
			delete_option( $option );
			return add_option( $option, $values );
		}
	}

	if( !function_exists( 'ipanel_get_google_fonts' ) ){
		function ipanel_get_google_fonts( $transient_time = 0 ){
			if( !is_numeric( $transient_time ) && 0 >= $transient_time )
				$transient_time = 2 * 24 * 60 * 60;
			$transient = get_transient( '_ipanel_google_font_' );
			if( $transient === false ){
				$data = wp_remote_get( 'http://phat-reaction.com/googlefonts.php?format=json' );
				if( is_wp_error( $data ) )
					return false;
				$data = json_decode( $data['body'] );
				$fonts = array();
				foreach( $data as $font )
					$fonts[ $font->{'css-name'} ] = $font->{'font-name'};
				set_transient( '_ipanel_google_font_', $fonts, $transient_time );
				return $fonts;
			} else {
				return $transient;
			}

		}
	}

	if( !function_exists('ipanel_get_categories') ){
		function ipanel_get_categories(){
			$categories = get_categories();
			$output = array();
			foreach ($categories as $cat) {
				$output[$cat->term_id] = $cat->name;
			}
			return $output;
		}
	}

	if( !function_exists( 'ipanel_get_tags' ) ){
		function ipanel_get_tags( $options = array() ){
			$tags = get_tags( (array) $options );
			$output = array();
			foreach ($tags as $tag) {
				$output[$tag->term_id] = $tag->name;
			}
			return $output;
		}
	}
	if( !function_exists( 'ipanel_get_users' ) ){
		function ipanel_get_users( $extra = array() ){
			if (!is_array($extra) || count($extra) == 0) {
				$extra = array(
					'orderby' => 'post_count',
					'order' => 'DESC'
				);
			}
			$output = array();
			$users = new WP_User_Query( (array) $extra );
			foreach ($users->results as $user) {
				$output[$user->data->ID] = $user->data->display_name;
			}
			return $output;
		}
	}

	if( !function_exists( 'ipanel_get_posts' ) ){
		function ipanel_get_posts( $extra = array() ){
			$posts = get_posts((array) $extra);
			$output = array();
			foreach ($posts as $post) {
				$output[$post->ID] = $post->post_title;
			}
			return $output;
		}
	}

	if( !function_exists( 'ipanel_get_pages' ) ){
		function ipanel_get_pages( $extra = array() ){
			$output = array();
			$pages = get_pages((array) $extra);
			foreach ($pages as $page) {
				$output[$page->ID] = $page->post_title;
			}
			return $output;
		}
	}

	if( !function_exists( 'ipanel_get_post_types' ) ){
		function ipanel_get_post_types( $extra = array() ){
			return get_post_types((array) $extra);
		}
	}

	if( !function_exists( 'ipanel_get_taxonomies' ) ){
		function ipanel_get_taxonomies( $extra = array() ){
			$taxonomies = get_taxonomies((array) $extra, 'objects');
			$output = array();
			foreach ($taxonomies as $taxonomy_name => $taxonomy_data) {
				$output[$taxonomy_name] = $taxonomy_data->labels->name;
			}
			return $output;
		}
	}
	if( !function_exists( 'ipanel_get_plugins' ) ){
		function ipanel_get_plugins(){
			$output = array();
			if (!function_exists('get_plugin_data')) {
				include_once ABSPATH . '/wp-admin/includes/plugin.php';
			}
			$plugins = get_option('active_plugins');
			foreach ($plugins as $plugin) {
				$plugin_data     = get_plugin_data(WP_PLUGIN_DIR . '/' . $plugin);
				$output[$plugin] = $plugin_data['Name'];
			}
			return $output;
		}
	}