<?php
	/*
	  Plugin Name: iPanel
	  Plugin URI: http://www.vahidd.com
	  Description: iPanel in plugin
	  Version: 2.0
	  Author: vahidd
	  Author URI: http://vahidd.com
	 */

	require_once dirname( __FILE__ ) . '/includes/common.php';
	if ( !class_exists( 'Load_iPanel' ) ) {

		class Load_iPanel{

			protected $_panels = array();

			/*
			 * Initialize function
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public static function init() {
				$class = __CLASS__;
				add_action( 'init', create_function( '', 'return new '. $class . ';' ) );
			}

			/*
			 * Constructor
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __construct() {
				$GLOBALS['ipanel_loaded'] = true;
				$this->_load_panels();
				if( empty( $this->_panels ) )
					return false;
				foreach( $this->_panels as $id => $panel ) {
					$GLOBALS['ipanel'][$id] = new iPanel( $id, $panel );
				}
				require_once dirname(__FILE__) . '/includes/classes/ipanel-ajax.php';
				new iPanel_AJAX();
				return true;
			}

			/*
			 * Load panels that are hooked to "ipanel_otions"
			 *
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			protected function _load_panels() {
				$panels = apply_filters( 'ipanel_options', array() );
				if( empty( $panels ) || !is_array( $panels ) )
					return false;
				foreach( $panels as $panel_id => $panel ){
					if(
						is_numeric( $panel_id ) 		 			// has no ID!
						|| empty( $panel['fields'] )				// fields are empty
						|| !is_array( $panel['fields'] )			// fields are not array
						|| empty( $panel['settings'] )				// settings is empty
						|| !is_array( $panel['settings'] )			// settings is not array
						|| empty( $panel['settings']['name'] )	 	// name in settings is empty
						|| !is_string( $panel['settings']['name'] )   // name in settings is string
					)
						continue;
					$this->_panels[ $panel_id ] = $panel;
				}
			}

		}
	}

	if ( !class_exists( 'iPanel' ) ) {
		class iPanel{


			protected $_id, $_panel, $values = array(), $text_domain = 'default';

			/*
			 * Constructor
			 *
			 * @param string
			 * @param array
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __construct( $id, &$panel ) {
				$this->_id 	  =  $id;
				$this->_panel =& $panel;
				$this->_setup_values();
				if( !empty( $panel['settings']['text_domain'] ) && is_string( $panel['settings']['text_domain'] ) )
					$this->text_domain = $panel['settings']['text_domain'];
				$this->_panel['settings']['text_domain'] = $this->text_domain;
				$this->values = stripslashes_deep( $this->values );
			}


			/*
			 * Setup values
			 *
			 * @since 2.0
			 * @access protected
			 * @return array
			 */
			protected function _setup_values() {

				if( isset( $this->_panel['settings']['values'] ) && ( ( is_string( $this->_panel['settings']['values'] ) && is_serialized( $this->_panel['settings']['values'] ) ) || is_array( $this->_panel['settings']['values'] ) ) )
					$option = 1;
				else if( isset( $this->_panel['settings']['option_name'] ) && is_string( $this->_panel['settings']['option_name'] ) )
					$option = $this->_panel['settings']['option_name'];
				else
					$option = 'ipanel_' . $this->_id;

				if( $option === 1 ){
					if( is_string( $this->_panel['settings']['values'] ) )
						$this->values = (array) @unserialize( $this->_panel['settings']['values'] );
					else
						$this->values = $this->_panel['settings']['values'];
				}
				else if( is_string( $option ) )
					$this->values = get_option( $option, array() );
				else
					$this->values = array();
			}


			/*
			 * Array push
			 *
			 * push item to specific position of array
			 *
			 * @param (array)
			 * @param (mixed)
			 * @param (int)
			 *
			 * @since 2.0
			 * @access public
			 * @return array
			 * @source http://stackoverflow.com/a/11321318
			 */
			public function array_push($array, $index, $val) {
				$temp = array();
				$size = count( $array );
				if ( !$index || $index < 0 || $index > $size ) {
					return false;
				}
				$temp = array_slice( $array, 0, $index );
				array_push( $temp, $val );
				$temp  = array_merge( $temp, array_slice( $array, $index, $size ) );
				$array = $temp;
				return $array;
			}


			/*
			 * Get settings item
			 *
			 * @param (mixed)
			 * @param (mixed)
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function settings_item( $item, $default = false ) {
				$settings =& $this->_panel['settings'];
				return array_key_exists( $item, $settings ) ? $settings[$item] : $default;
			}


			/*
			 * Menu callback
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function callback() {
				require_once 'includes/classes/ipanel-template.php';
				$GLOBALS['is_ipanel'] = true;
				$template = new iPanel_Template( $this->_id, $this->_panel, $this->values );
				return $template->display();
			}


			/*
			 * Load ipanl assets
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function load() {
				$screen = get_current_screen();
				$id = $this->_id;
				$help_tabs = array();

				if( !empty( $this->_panel['settings']['help_tabs'] ) && is_array( $this->_panel['settings']['help_tabs'] ) ){
					foreach( $this->_panel['settings']['help_tabs'] as $i => $_tab ){
						if(
							is_array( $_tab )
							&& !empty( $_tab['title'] )
							&& is_string( $_tab['title'] )
							&& !empty( $_tab['content'] )
							&& is_string( $_tab['content'] )
						){
							$_tab['id'] = $id . '-help-tab-' . $i;
							$help_tabs[] = $_tab;
						}

					}
				}
				foreach( $help_tabs as $tab ){
					$screen->add_help_tab( array(
						'id'	=> $tab['id'],
						'title'	=> $tab['title'],
						'content'	=> '<p>' . $tab['content'] . '</p>',
					) );
				}

				add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_assets' ) );
			}

			/*
			 * Enqueue admin assets
			 *
			 * @param (type) about this param
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function enqueue_admin_assets() {
				$url = $this->_panel['settings']['url'] . 'assets/';
				wp_enqueue_script( 'wp-color-picker' );
				wp_enqueue_script( 'jquery-form' );
				wp_enqueue_script( 'jquery-ui-core' );
				wp_enqueue_script( 'jquery-ui-datepicker' );
				wp_enqueue_script( 'jquery-ui-sortable' );
				wp_enqueue_script( 'jquery-ui-slider' );


				$items = array(
					'panel_id' => $this->_id,
					'is_saved' => (int) get_option( ipanel_get_panel_option_name( $this->_id ), false ),
					'messages' => array(
						'confirm' => __( 'Are you sure?', $this->text_domain ),
						'error' => __( 'An error occurred.', $this->text_domain ),
						'error_try_again' => __( 'An error occurred, please try again later.', $this->text_domain )
					),
					'ajax_url' => admin_url('admin-ajax.php'),
					'nonce' => wp_create_nonce( 'ipanel' )
				);

				wp_enqueue_script( 'ipanel-plugins-js', $url . 'js/plugins.js' );
				wp_enqueue_script( 'ipanel-general-js', $url . 'js/general.js' );
				wp_localize_script( 'ipanel-general-js', 'ipanel', $items );

				wp_enqueue_script( 'ipanel-ace-js', $url . 'plugins/ace/src-min-noconflict/ace.js' );


				wp_enqueue_script( 'ipanel-select2-js', $url . 'plugins/select2/select2.js' );
				wp_enqueue_style( 'ipanel-select2-css', $url . 'plugins/select2/select2.css' );


				wp_enqueue_style( 'wp-color-picker' );
				if( function_exists( 'wp_enqueue_media' ) ){
					wp_enqueue_media();
				} else {
					wp_enqueue_style('thickbox');
					wp_enqueue_script('media-upload');
					wp_enqueue_script('thickbox');
				}

			}


			/*
			 * Echoes Menu callback
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function echo_callback() {
				echo $this->callback();
			}



			/*
			 * Get magic method
			 *
			 * This function makes the protected and private properties accessible outside of class
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function __get( $pro ){
				if( property_exists( $this, $pro ) )
					return $this->$pro;
				throw new Exception;
			}

			/*
			 * Get option
			 *
			 * @param (string) Option ID
			 * @param (mixed)  Default value
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function get_option( $option_id, $default = false ) {
				$option_name = ( isset( $this->_panel['settings']['option_name'] ) && is_string( $this->_panel['settings']['option_name'] ) ) ? $this->_panel['settings']['option_name'] : 'ipanel_' . $this->_id;
				$option = get_option( $option_name, false );
				if( is_array( $option ) && array_key_exists( $option_id, $option ) )
					return $option[$option_id];
				return $default;
			}
		}
	}


	if( !isset( $GLOBALS['ipanel_loaded'] ) )
		Load_iPanel::init();
