<?php
	if( !class_exists( 'iPanel_AJAX' ) ){

		class iPanel_AJAX{

			public $text_domain = 'default';
		
			/*
			 * Construct function
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __construct() {
				add_action( 'wp_ajax_ipanel', array( $this, 'callback' ) );
			}

			/*
			 * AJAX callback
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function callback() {

				$requests = array( 'save_panel', 'reset_panel', 'upload_file', 'download_export', 'get_export_code', 'import_panel' );
				$response = array();
				$response['status'] = 0;
				$has_error = false;

				if(	@$_SERVER['REQUEST_METHOD'] != 'POST' )
					$this->_die( $response );

				if( empty( $_POST['action2'] ) || !is_string( $_POST['action2'] ) || !in_array( $_POST['action2'], $requests ) ) {
					$response['msg'] = __( 'Invalid request.' );
					$this->_die( $response );
				}

				if( empty( $_POST['_nonce'] ) || !is_string( $_POST['_nonce'] ) || !wp_verify_nonce( $_POST['_nonce'], 'ipanel' ) ) {
					$response['msg'] = __( 'Error in nonce.' );
					$this->_die( $response );
				}

				if( empty( $_POST['panel_id'] ) || !is_string( $_POST['panel_id'] ) || !array_key_exists( $_POST['panel_id'], @ (array) $GLOBALS['ipanel'] ) ) {
					$response['msg'] = __( 'Error in panel ID.' );
					$this->_die( $response );
				}

				if( !array_key_exists( $_POST['panel_id'], @ (array) $GLOBALS['ipanel'] ) || !is_object( @$GLOBALS['ipanel'][$_POST['panel_id']] ) ){
					$response['msg'] = __( 'There is no panel with submitted ID.' );
					$this->_die( $response );
				}


				try{
					call_user_func( array( $this, $_POST['action2'] ) );
				}
				catch( Exception $e ){
					$this->_die(
						array(
							'status' => 0,
							'msg'    => $e->getMessage()
						)
					);
				}
				$this->_die( $response );
			}

			/*
			 * Checks whether the request is an AJAX request
			 *
			 * @since 2.0
			 * @access public
			 * @return bool
			 */
			public function is_ajax() {
				return ( isset( $_SERVER['HTTP_X_REQUESTED_WITH'] ) && strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] ) == 'xmlhttprequest' );
			}

			/*
			 * Custom die
			 *
			 * @param (array) Custom parameters
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function _die( array $error_params = array( 'status' => 0 ) ) {
				empty( $error_params['status'] ) && $error_params['status'] = 0;
				die(
					json_encode(
						$error_params
					)
				);
			}

			/*
			 * Save panel callback
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function save_panel() {
				$panel_id = $_POST['panel_id'];
				if( empty( $_POST['data'] ) )
					throw new Exception( __( 'Data field is empty!' ) );
				$option_name = ipanel_get_panel_option_name( $panel_id );
				do_action( 'ipanel_save_panel', $panel_id );
				delete_option( $option_name );
				$update = update_option( $option_name, $_POST['data'] );

				if( !$this->is_ajax() ) {
					$redirect_params = array(
						'status' => '0'
					);
					if( $update )
						$redirect_params['status'] = '1';
					wp_redirect(
						add_query_arg(
							$redirect_params,
							$_SERVER['HTTP_REFERER']
						)
					);
					die;
				}
				if( $update )
					$this->_die(
						array(
							'status' => 1,
							'msg' 	 => __( 'Options Updated.' )
						)
					);
				else
					throw new Exception( __( 'An error happened. Please try again.' ) );
			}


			/*
			 * File uploader callback
			 *
			 * @param (array) Custom parameters
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function upload_file( array $error_params = array( 'status' => 0 ) ) {
				if( empty( $_POST['field_name'] ) || !is_string( $_POST['field_name'] ) )
					throw new Exception( __( 'File name filed is empty!' ) );
				$name = sanitize_text_field( $_POST['field_name'] );
				if ( !function_exists( 'wp_handle_upload' ) )
					require_once( ABSPATH . 'wp-admin/includes/file.php' );
				if( empty( $_FILES[ $name ] ) )
					throw new Exception( __( 'Upload field is empty!' ) );
				$name = $_FILES[ $name ];
				$upload_overrides = array( 'test_form' => false );
				$return_data = array();
				$multiple  = false;
				if( is_array( $name['error'] ) ){
					file_put_contents( 'print_r.txt', print_r( $_FILES, true ) );
					$multiple = true;

				}

				if( $multiple ){
					foreach( $name['name'] as $i => $_name_ ){
						$upload_array = array(
							'name' 	   => $name['name'][$i],
							'type'	   => $name['type'][$i],
							'tmp_name' => $name['tmp_name'][$i],
							'error'	   => $name['error'][$i],
							'size' 	   => $name['size'][$i]
						);
						$up = wp_handle_upload( $upload_array, $upload_overrides );
						if( isset( $up['file'] ) )
							$return_data[] = $up + array(
									'status' => 1,
									'name' => $name['name'][$i],
									'msg' => sprintf( __( '%s successfully uploaded.' ), '<strong>'.$name['name'][$i].'</strong>' )
								);
						else
							$return_data[] = array(
								'status' => 0,
								'name' => $name['name'][$i],
								'data' => $up,
								'msg' => sprintf( __( 'There was an error with uploading %s. %s' ), '<strong>'.$name['name'][$i].'</strong>', $up['error'] )
							);
					}
				}
				else {
					$return_data[] = wp_handle_upload( $name, $upload_overrides );
				}

				$this->_die(
					array(
						'status' => 1,
						'data' 	 => $return_data,
						'msg' 	 => __( 'File was uploaded.' )
					)
				);
			}


			/*
			 * Download export handler
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function download_export() {
				$panel_id = $_POST['panel_id'];

				$settings = ipanel_get_panel_settings( $panel_id );
				$data = get_option( ipanel_get_panel_option_name( $panel_id ), array() );
				$data = base64_encode( serialize( $data ) );
				$file_name = $settings['name'];
				$file_name .= ' - ';
				$file_name .= current_time( 'mysql' );
				$file_name .= '.txt';

				// No Cache
				header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');
				header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
				header('Cache-Control: no-store, no-cache, must-revalidate');
				header('Cache-Control: post-check=0, pre-check=0', false);
				header('Pragma: no-cache');

				header('Content-Type: application/force-download');
				header("Content-Length: " . strlen($data));
				header('Content-Disposition: attachment; filename="' . $file_name . '"');

				die( $data );
			}

			/*
			 * Get export code callback
			 *
			 *
			 *
			 * @param (type) about this param
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function get_export_code() {
				$panel_id = $_POST['panel_id'];
				$data = get_option( ipanel_get_panel_option_name( $panel_id ), array() );
				$data = base64_encode( serialize( $data ) );
				$this->_die(
					array(
						'status' => 1,
						'data' => $data
					)
				);
			}


			/*
			 * Import panel callback
			 *
			 * @since 2.0
			 * @access public
			 * @return mixed
			 */
			public function import_panel(){
				$panel_id = $_POST['panel_id'];
				$option_name = ipanel_get_panel_option_name( $panel_id );
				$import_types = array( 'file', 'code' );
				$redirect_time = 5000;
				if( empty( $_POST['import_type'] ) || !is_string( $_POST['import_type'] ) || !in_array( $_POST['import_type'], $import_types ) )
					$this->_die(
						array(
							'status' => 0,
							'msg' => __( 'Import type field is empty!' )
						)
					);
				$content = '';
				switch( $_POST['import_type'] ){
					case( 'file' ):
						$content = file_get_contents( $_FILES['file']['tmp_name'] );
						break;
					case( 'code' ):
						$content = $_POST['code'];
						break;
				}
				$content = base64_decode( $content );
				$content = @ (array) unserialize( $content );
				delete_option( $option_name );
				add_option( $option_name, $content );
				$this->_die( array(
					'status' => 1,
					'redirect_after' => $redirect_time,
					'msg' => __( sprintf( 'Panel imported. The page will be reloaded in %d seconds.', $redirect_time / 1000 ) )
				) );

			}

			/*
			 * Reset panel callback
			 *
			 * @since 2.0
			 * @access protected
			 * @return void
			 */
			protected function reset_panel(){
				$panel_id = $_POST['panel_id'];
				$reload_time = 5000;
				if( ipanel_restore_panel_default_values( $panel_id ) )
					$this->_die(
						array(
							'status' => 1,
							'msg' 	 => __( sprintf( 'Options Reset. The page will be reloaded in %d seconds.', $reload_time / 1000 ) ),
							'reload_time' => $reload_time
						)
					);
				else
					throw new Exception( __( 'An error occurred, please try again later.' ) );
			}

		}

	}