<?php
	if( !class_exists( 'iPanel_Fields' ) ) {
		abstract class iPanel_Fields{

			protected $_supported_fields = array(
				'text', 'textarea', 'select', 'wp_editor', 'repeater', 'color', 'wp_color', 'checkbox', 'image_checkbox', 'multicheckbox', 'multiselect', 'radio', 'image_radio',
				'media', 'ajax_uploader', 'code', 'typography', 'slider', 'date', 'sorter'
			);

			abstract function field_name( &$field );

			/*
			 * Generate HTML attributes from array
			 *
			 * @param (array) attributes
			 * @param (string)
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			protected function _generate_attributes_string( array $attrs, $d = '"' ){
				$output = '';
				foreach( $attrs as $attr => $val )
					$output .= "{$attr}={$d}{$val}{$d} ";
				return trim( $output );
			}

			/*
			 * Field wrapper
			 *
			 * @param (string) Input string
			 * @param (array)  Field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function section( $input, $field ) {
				$class  = 'ipanel-section ipanel-nonrepeater-section';
				$class .= ' ' . $field['type'];
				$class .= !empty( $field['class'] ) && is_string( $field['class'] ) ? ' ' . $field['class'] : '';
				$logics = array();
				if( !empty( $field['conditional_logic'] ) && is_array( $field['conditional_logic'] ) )
					foreach( $field['conditional_logic'] as $logic ){
						if( empty( $logic['value'] ) || is_array( $logic['value'] ) || is_object( $logic['value'] ) )
							continue;
						$logics[] = $logic;
					}
				$class .= !empty( $logics ) ? ' has-conditional-logic' : '';

				$attrs = array();
				$attrs['class'] = $class;
				$attrs['data-id'] = !empty( $field['id'] ) && is_string( $field['id'] ) ? $field['id'] : '';
				$attrs['data-type'] = $field['type'];

				if( !empty( $logics ) )
					$attrs['data-conditional-logics'] = json_encode($logics);

				ob_start();
				?>
				<div <?php echo $this->_generate_attributes_string( $attrs, "'" ); ?>>
					<div class="ipanel-heading">
						<h3><?php echo $field[ 'name' ]; ?></h3>
						<?php echo !empty( $input[ 'after-header' ] ) && is_string( $input['after-header'] ) ? $input['after-header'] : ''; ?>
					</div>
					<div class="ipanel-controls">
						<?php echo $input['input']; ?>
					</div>
					<?php
						if( !empty( $field['desc'] ) && is_string( $field['desc'] ) ) :

							$desc_in_tooltip = false;

							if( isset( $this->_panel['settings']['desc_in_icon'] ) && $this->_panel['settings']['desc_in_icon'] )
								$desc_in_tooltip = true;

							if( isset( $field['desc_in_icon'] ) && !$field['desc_in_icon'] )
								$desc_in_tooltip = false;
							?>
						<div class="ipanel-desc <?php echo $desc_in_tooltip ? 'tooltip' : ''; ?>">
							<?php
								if( $desc_in_tooltip )
									echo '<i></i>';
								echo '<p>'.$field[ 'desc' ].'</p>';
							?>
						</div>
					<?php endif; ?>
				</div>
				<?php
				return ob_get_clean();
			}


			/*
			 * Repeater Section
			 *
			 * @param (string) Input string
			 * @param (array)  Field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function repeater_section( $input, $field ) {
				ob_start();
				?>
				<div class="ipanel-section ipanel-repeater-section <?php echo $field['type']; ?>" data-id="<?php echo @$field['id']; ?>">
					<div class="ipanel-heading ipanel-repeater-heading">
						<h3><?php echo $field[ 'name' ]; ?></h3>
					</div>
					<div class="ipanel-controls ipanel-repeater-controls">
						<?php echo $input['input']; ?>
					</div>
					<?php if( !empty( $field['desc'] ) && is_string( $field['desc'] ) ) : ?>
						<div class="ipanel-desc ipanel-repeater-desc">
							<?php echo $field[ 'desc' ]; ?>
						</div>
					<?php endif; ?>
				</div>
				<?php
				return ob_get_clean();
			}


			/*
			 * Call
			 *
			 * Call to specific field
			 *
			 * @param (string) name of field
			 * @param (array)
			 *
			 * @since 2.0
			 * @access public
			 * @return bool|string
			 */
			public function call_field( $field, $params ) {
				if( !in_array( $field, $this->_supported_fields ) )
					return false;
				return call_user_func( array( $this, "__{$field}__"), $params );
			}


			/*
			 * Filter fields array
			 *
			 * @param (array) fields array
			 *
			 * @since 2.0
			 * @access public
			 * @return bool|array
			 */
			public function filter_fields( &$fields, $allowed_fields = array(), $ignore = array() ) {
				if( !is_array( $fields ) )
					return false;
				$output = false;
				if( empty( $allowed_fields ) )
					$allowed_fields = $this->_supported_fields;

				foreach( $fields as $field ){
					if( in_array( $field['type'], $ignore ) ){
						$output[] = $field;
						continue;
					}
					if(
						   empty( $field['id'] )   || !is_string( $field['id'] )
						|| empty( $field['name'] ) || !is_string( $field['name'] )
						|| empty( $field['type'] ) || !is_string( $field['type'] )
						|| !in_array( $field['type'], $allowed_fields )
						|| $field['type'] == 'repeater' && ( empty( $field['options'] ) || !is_array( $field['options'] )  )
					)
					{
						continue;
					}
					$output[] = $field;
				}

				return $output;
			}


			/*
			 * Generate fields HTML
			 *
			 * @param (array)
			 * @param (string)
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function generate_fields( &$fields, $section_callback = 'section', &$values = array(), $filter_fields = true ){

				$output = '';
				$tab_index = 0;
				!is_array( $values ) && $values = array();
				if( $filter_fields )
					$fields = $this->filter_fields(
						$fields,
						array(),
						array(
							'tab',
							'subtab',
							'open_section',
							'close_section',
							'info'
						)
					);
				if( empty( $fields ) )
					return '';
				foreach( $fields as $field ){
					if( empty( $field['type'] ) || !is_string( $field['type'] ) )
						continue;
					if( $field['type'] == 'tab' || $field['type'] == 'subtab' ){
						if( $tab_index++ > 0 )
							$output .= '</div>';
						$output .= '<div class="ipanel-group" data-id="'.$field['id'].'">';
						continue;
					}
					if( $field['type'] == 'open_section' ){
						$output .= '<div class="ipanel-section-container">';
						$output .= "<h3>{$field['name']}</h3>";
						$show_body = false;
						if( isset( $field['show-body'] ) && $field['show-body'] )
							$show_body = true;
						$output .= '<div class="section-body" style="display: '.( $show_body ? 'block' : 'none' ).'">';
						continue;
					}
					if( $field['type'] == 'close_section' ){
						$output .= '</div>';
						$output .= '</div>';
						continue;
					}
					if( $field['type'] == 'info' ){
						$default_styles = array(
							'danger',
							'success',
							'info',
							'alert'
						);
						$style = !empty( $field['style'] ) && is_string( $field['style'] ) && in_array( $field['style'], $default_styles ) ? $field['style'] : 'info';
						if( !empty( $field['text'] ) && is_string( $field['text'] ) )
							$output .= "<div class='ipanel-info {$style} ".( !empty( $field['class'] ) && is_string( $field['class'] ) ? $field['class'] : '' )."' ".( !empty( $field['id'] ) && is_string( $field['id'] ) ? 'data-id="'.$field['id'].'"' : '' )."><p>{$field['text']}</p></div>";
						continue;
					}
					$field['value'] = isset( $field['value'] ) ? $field['value'] : ( array_key_exists( $field['id'], $values ) ? $values[$field['id']] : null );
					if( !$field_string = $this->call_field( $field['type'], $field ) ) {
						$output .= '<!-- "'.$field['type'].'" type does not supported! -->';
						continue;
					}
					$callback = isset( $this->_panel['settings']['section_callback'] ) && is_callable( $this->_panel['settings']['section_callback'] )
						? $this->_panel['settings']['section_callback']
						: array( $this, $section_callback );
					$callback_params = array( $field_string, $field );
					$output .= call_user_func_array( $callback, $callback_params );


				}
				$tab_index > 0 && $output .= '</div>';
				return $output;
			}

			/*
			 * Text
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __text__( $field ) {
				$attrs = array();
				if( !empty( $field['value'] ) )
					$attrs['value'] = $field['value'];
				$attrs['name'] = $this->field_name( $field );
				$attrs['type'] = 'text';
				$width = !empty( $field['width'] ) && is_string( $field['width'] ) ? $field['width'] : '40%';
				$attrs['style'] = "width:{$width};";
				$has_prefix = false;
				if( !empty( $field['prefix'] ) && is_string( $field['prefix'] ) )
					$has_prefix = true;
				$has_suffix = false;
				if( !empty( $field['suffix'] ) && is_string( $field['suffix'] ) )
					$has_suffix = true;
				ob_start();
				?>
				<label class="ipanel-text-field-container <?php echo $has_prefix ? ' has-prefix' : ''; echo $has_suffix ? ' has-suffix' : ''; ?>">
					<?php
						if( !empty( $field['prefix'] ) && is_string( $field['prefix'] ) ){
							echo '<span class="ipanel-prefix">'.$field['prefix'].'</span>';
						}
					?>
					<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
					<?php
						if( !empty( $field['suffix'] ) && is_string( $field['suffix'] ) ){
							echo '<span class="ipanel-suffix">'.$field['suffix'].'</span>';
						}
					?>
				</label>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Textarea
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __textarea__( $field ) {
				$attrs = array();
				$attrs['name'] = $this->field_name( $field );
				$attrs['rows'] = !empty( $field['rows'] ) && is_string( $field['rows'] ) ? $field['rows'] : '3';
				$attrs['cols'] = !empty( $field['cols'] ) && is_string( $field['cols'] ) ? $field['cols'] : '50';
				ob_start();
				?>
				<textarea <?php echo $this->_generate_attributes_string( $attrs ); ?>><?php echo ( !empty( $field['value'] ) && is_string( $field['value'] ) ) ? $field['value'] : ''; ?></textarea>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Typography
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __typography__( $field ) {
				$output = '';

				$typo_fields = array(
					'font-size' => true,
					'color' => true,
					'font-family' => true,
					'font-style' => true
				);
				if( isset( $field['color'] ) && $field['color'] == false )
					$typo_fields['color'] = false;
				if( isset( $field['font-style'] ) && $field['font-style'] == false )
					$typo_fields['font-style'] = false;
				if( isset( $field['font-family'] ) && $field['font-family'] == false )
					$typo_fields['font-family'] = false;
				if( isset( $field['font-size'] ) && $field['font-size'] == false )
					$typo_fields['font-size'] = false;

				ob_start();

				?>
					<div class="ipanel-typography-container">
						<?php
							if( $typo_fields['color'] ) {
								$field['__type__'] = 'color';
								$attrs = array(
									'type' => 'text',
									'class' => 'ipanel-color',
									'name' => $this->field_name( $field ),
									'value' => !empty( $field['value']['color'] ) && is_string( $field['value']['color'] ) ? $field['value']['color'] : ''
								);
						?>
						<div class="color">
							<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
						</div>
						<?php } ?>
						<?php
							if( $typo_fields['font-style'] ) {
								$field['__type__'] = 'font-style';
								$options = !empty( $field['font-style-options'] ) && is_array( $field['font-style-options'] ) ? $field['font-style-options'] : array(
									'normal' => __( 'Normal', $this->_panel['settings']['text_domain'] ),
									'italic' => __( 'Italic', $this->_panel['settings']['text_domain'] ),
									'oblique' => __( 'Oblique', $this->_panel['settings']['text_domain'] ),
									'initial' => __( 'Initial', $this->_panel['settings']['text_domain'] ),
									'inherit' => __( 'Inherit', $this->_panel['settings']['text_domain'] )
								);
								$value = !empty( $field['value']['font-style'] ) && is_string( $field['value']['font-style'] ) ? $field['value']['font-style'] : null;
								$attrs = array(
									'class' => 'ipanel-typography-font-style',
									'name' => $this->field_name( $field ),
									'style' => 'width:200px;',
									'value' => !empty( $field['value']['font-style'] ) && is_string( $field['value']['font-style'] ) ? $field['value']['font-style'] : ''
								);
						?>
						<div class="font-style">
							<select <?php echo $this->_generate_attributes_string( $attrs ); ?>>
								<?php foreach( $options as $option_key => $option_value ){?>
								<option value="<?php echo $option_key; ?>" <?php selected( $value, $option_key ); ?>><?php echo $option_value; ?></option>
								<?php } ?>
							</select>
						</div>
						<?php } ?>
						<?php
							if( $typo_fields['font-size'] ) {
								$field['__type__'] = 'font-size';
								$options = !empty( $field['font-size-options'] ) && is_array( $field['font-size-options'] ) ? $field['font-size-options'] : array(
									'12px' => '12px',
									'13px' => '13px',
									'14px' => '14px',
									'15px' => '15px',
									'16px' => '16px',
									'17px' => '17px',
									'18px' => '18px',
									'19px' => '19px',
									'20px' => '20px',
									'21px' => '21px',
									'22px' => '22px',
									'23px' => '23px',
									'24px' => '24px',
									'25px' => '25px',
									'26px' => '26px',
									'27px' => '27px',
									'28px' => '28px',
									'29px' => '29px',
									'30px' => '30px',
									'31px' => '31px',
									'32px' => '32px'
								);
								$value = !empty( $field['value']['font-size'] ) && is_string( $field['value']['font-size'] ) ? $field['value']['font-size'] : null;
								$attrs = array(
									'class' => 'ipanel-typography-font-size',
									'name' => $this->field_name( $field ),
									'style' => 'width:200px;',
									'value' => !empty( $field['value']['font-size'] ) && is_string( $field['value']['font-size'] ) ? $field['value']['font-size'] : ''
								);
						?>
						<div class="font-size">
							<select <?php echo $this->_generate_attributes_string( $attrs ); ?>>
								<?php foreach( $options as $option_key => $option_value ){?>
								<option value="<?php echo $option_key; ?>" <?php selected( $value, $option_key ); ?>><?php echo $option_value; ?></option>
								<?php } ?>
							</select>
						</div>
						<?php } ?>

						<?php
							if( $typo_fields['font-family'] ) {
								$field['__type__'] = 'font-family';
								$options = !empty( $field['font-family-options'] ) && is_array( $field['font-family-options'] ) ? $field['font-family-options'] : ipanel_get_google_fonts();
								$value = !empty( $field['value']['font-family'] ) && is_string( $field['value']['font-family'] ) ? $field['value']['font-family'] : null;
								$attrs = array(
									'class' => 'ipanel-typography-font-family',
									'name' => $this->field_name( $field ),
									'style' => 'width:200px;',
									'value' => !empty( $field['value']['font-family'] ) && is_string( $field['value']['font-family'] ) ? $field['value']['font-family'] : ''
								);
						?>
						<div class="font-family">
							<select <?php echo $this->_generate_attributes_string( $attrs ); ?>>
								<?php foreach( $options as $option_key => $option_value ){?>
								<option value="<?php echo $option_key; ?>" <?php selected( $value, $option_key ); ?>><?php echo $option_value; ?></option>
								<?php } ?>
							</select>
						</div>
						<?php } ?>

					</div>
				<div class="clr"></div>
				<?php
				$output .= ob_get_clean();
				return array( 'input' => $output );
			}



			/*
			 * Code Field
			 *
			 * @param (array) Field array
			 *
			 * @since 2.0
			 * @access public
			 * @return void
			 */
			public function __code__( $field ) {

				$themes = array(
					'ambiance',
					'chaos',
					'chrome',
					'clouds',
					'clouds_midnight',
					'cobalt',
					'dawn',
					'dreamweaver',
					'eclipse',
					'github',
					'idle_fingers',
					'katzenmilch',
					'kr',
					'kuroir',
					'merbivore',
					'merbivore_soft',
					'mono_industrial',
					'monokai',
					'pastel_on_dark',
					'solarized_dar',
					'solarized_light',
					'terminal',
					'textmate',
					'tomorrow',
					'tomorrow_night',
					'tomorrow_night_blue',
					'tomorrow_night_bright',
					'tomorrow_night_eighties',
					'twilight',
					'vibrant_ink',
					'xcode'
				);
				$langs  = array(
					'abap',
					'actionscript',
					'ada',
					'apache_conf',
					'asciidoc',
					'assembly_x86',
					'autohotkey',
					'batchfile',
					'c9search',
					'c_cpp',
					'cirru',
					'clojure',
					'cobol',
					'coffee',
					'coldfusion',
					'csharp',
					'css',
					'curly',
					'd',
					'dart',
					'diff',
					'django',
					'dot',
					'ejs',
					'erlang',
					'forth',
					'ftl',
					'gherkin',
					'glsl',
					'golang',
					'groovy',
					'haml',
					'handlebars',
					'haskell',
					'haxe',
					'html',
					'html_completions',
					'html_ruby',
					'ini',
					'jack',
					'jade',
					'java',
					'javascript',
					'json',
					'jsoniq',
					'jsp',
					'jsx',
					'julia',
					'latex',
					'less',
					'liquid',
					'lisp',
					'livescript',
					'logiql',
					'lsl',
					'lua',
					'luapage',
					'lucene',
					'makefile',
					'markdown',
					'matlab',
					'mel',
					'mushcode',
					'mushcode_high_rules',
					'mysql',
					'nix',
					'objectivec',
					'ocaml',
					'pascal',
					'perl',
					'pgsql',
					'php',
					'plain_text',
					'powershell',
					'prolog',
					'properties',
					'protobuf',
					'python',
					'r',
					'rdoc',
					'rhtml',
					'ruby',
					'rust',
					'sass',
					'scad',
					'scala',
					'scheme',
					'scss',
					'sh',
					'sjs',
					'smarty',
					'snippets',
					'soy_template',
					'space',
					'sql',
					'stylus',
					'svg',
					'tcl',
					'tex',
					'text',
					'textile',
					'toml',
					'twig',
					'typescript',
					'vbscript',
					'velocity',
					'verilog',
					'vhdl',
					'xml',
					'xquery',
					'yaml'
				);
				$lang   = !empty( $field['lang'] )   && is_string( $field['lang'] ) && in_array( $field['lang'], $langs ) ? $field['lang'] : 'html';
				$width  = !empty( $field['width'] )  && is_string( $field['width'] ) ? $field['width'] : '100%';
				$height = !empty( $field['height'] ) && is_string( $field['height'] ) ? $field['height'] : '150px';

				$attrs = array();
				$attrs['style'] = "width:{$width};height:{$height}";
				$attrs['data-options'] = json_encode( array(
					'theme' => !empty( $field['theme'] ) && is_string( $field['theme'] ) && in_array( $field['theme'], $themes ) ? $field['theme'] : 'twilight',
					'lang' => $lang
				) );
				ob_start();
				?>
				<script src="<?php echo $this->_panel['settings']['url']; ?>/assets/plugins/ace/src-min-noconflict/mode-<?php echo $lang; ?>.js"></script>
				<div class="ipanel-code-wrapper">
					<textarea name="<?php echo $this->field_name( $field ); ?>" style="display: none" class="fuck"><?php echo $field['value']; ?></textarea>
					<div class="ipanel-code" <?php echo $this->_generate_attributes_string( $attrs, "" ); ?>></div>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}



			/*
			 * Checkbox
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __checkbox__( $field ) {
				$attrs = array();
				if( !empty( $field['value'] ) && $field['value'] == 1 )
					$attrs['checked'] = 'checked';
				$attrs['value'] = '1';
				$attrs['name'] = $this->field_name( $field );
				$attrs['type'] = 'checkbox';
				if( isset( $field['label'] ) && is_string( $field['label'] ) )
					$label = $field['label'];
				else
					$label = '';
				ob_start();
				?>
				<label>
					<input <?php echo $this->_generate_attributes_string($attrs); ?>/>
					<?php echo $label; ?>
				</label>
				<?php
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Multi Checkbox
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __multicheckbox__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return 'Something is wrong with "'.$field['id'].'" options.';
				ob_start();
				foreach( $field['options'] as $option_id => $option_name ) {
					$attrs 					= array();
					$attrs['name']  		= $this->field_name( $field );
					$attrs['type']  		= 'checkbox';
					$attrs['value'] 		= $option_id;
					if( isset( $field['value'] ) && is_array( $field['value'] ) && in_array( $option_id, $field['value'] ) )
						$attrs['checked'] = 'checked';
					?>
					<label class="ipanel-multicheckbox-option">
						<input <?php echo $this->_generate_attributes_string($attrs); ?>/>
						<?php echo $option_name; ?>
					</label>
					<?php
				}
				return array( 'input' => ob_get_clean() );
			}



			/*
			 * Image Checkbox
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __image_checkbox__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return 'Something is wrong with "'.$field['id'].'" options.';
				ob_start();
				foreach( $field['options'] as $option_id => $option ) {

					if( empty( $option['label'] ) || !is_string( $option['image'] ) || empty( $option['label'] ) || !is_string( $option['label'] )  )
						continue;

					$attrs 			= array();
					$attrs['name']  = $this->field_name( $field );
					$attrs['type']  = 'checkbox';
					$attrs['value'] = $option_id;
					$cheked 		= isset( $field['value'] ) && is_array( $field['value'] ) && in_array( $option_id, $field['value'] );

					if( $cheked )
						$attrs['checked'] = 'checked';
					?>
					<label class="ipanel-image-multicheckbox-option <?php echo ( $cheked ? 'checked' : 'unchecked' ); ?>">
						<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
						<img src="<?php echo $option['image']; ?>" alt="<?php echo $option['label']; ?>" title="<?php echo $option['label']; ?>"/>
						<i></i>
					</label>
					<?php

				}
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Color Picker
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __color__( $field ) {
				$attrs = array();
				if( !empty( $field['value'] ) )
					$attrs['value'] = $field['value'];
				$attrs['name']  	= $this->field_name( $field );
				$attrs['type']  	= 'text';
				$attrs['class'] 	= 'ipanel-color';
				ob_start();
				?>
				<div class="ipanel-color-field-wrapper">
					<div class="preview" style="background-color: <?php echo ( empty( $field['value'] ) ? '' : $field['value'] ); ?>"></div>
					<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Color Picker
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __wp_color__( $field ) {
				$attrs = array();
				if( !empty( $field['value'] ) )
					$attrs['value'] = $field['value'];
				$attrs['name']  	= $this->field_name( $field );
				$attrs['type']  	= 'text';
				$attrs['class'] 	= 'ipanel-wpcolor';
				ob_start();
				?>
				<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
				<?php
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Repeater
			 *
			 * @param (type) about this param
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __repeater__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return '<!-- ('.$field['id'].') Repeater options are empty! -->';

				$output			= '';
				$values 		= isset( $field['value'] ) && is_array( $field['value'] ) ? $field['value'] : array(0);
				$repeater_index = 0;

				$title = !empty( $field['title'] ) && is_string( $field['title'] ) ? $field['title'] : __( 'Item %d', $this->_panel['settings']['text_domain'] );

				$allowed_fields = $this->_supported_fields;
				unset( $allowed_fields[array_search('repeater', $this->_supported_fields)], $allowed_fields[array_search('wp_editor', $this->_supported_fields)] );
				$fields = $this->filter_fields( $field['options'], $allowed_fields );
				if( empty( $fields ) )
					return '<!-- Repeater does not contain valid options! -->';

				foreach( $fields as $i => $_field ){
					$fields[$i]['repeater_item'] = true;
					$fields[$i]['repeater_id']   = $field['id'];
				}

				$header_fields  = '<div class="open-close-repeater">';
				$header_fields .= '<a href="#" class="open-repeater-body">Open All</a>';
				$header_fields .= '<a href="#" class="close-repeater-body">Close All</a>';
				$header_fields .= '</div>';

				foreach( $values as $_val ){
					$output .= '<div class="ipanel-repeater-item">';
					$repeater_index++;
					ob_start();
					?>
					<div class="repeater-item-header">
						<?php echo sprintf( $title, $repeater_index ); ?>
						<div class="ipanel-repeater-options">
							<div class="ipanel-repeater-control" data-id="sortable-handle"></div>
							<div class="ipanel-repeater-control" data-id="duplicate-repeater"></div>
							<div class="ipanel-repeater-control" data-id="remove-item"></div>
						</div>
					</div>
					<div class="repeater-body">

					<?php
					$output .= ob_get_clean();
					foreach( $fields as $i => $_field ){
						$fields[$i]['repeater_pos'] = $repeater_index;
					}
					$output .= $this->generate_fields( $fields, 'repeater_section', $_val, false );
					$output .= '</div>';
					$output .= '</div>';
				}

				return array( 'input' => $output, 'after-header' => $header_fields );
			}



			/*
			 * Wordpress TinyMCE Editor
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __wp_editor__( $field ) {
				$attrs = array();
				$settings = ( !empty( $field['settings'] ) && is_array( $field['settings'] ) ) ? $field['settings'] : array();
				$content = ( !empty( $field['value'] ) && is_string( $field['value'] ) ) ? $field['value'] : '';
				ob_start();
				echo '<div class="ipanel-wp-editor-container" data-field-name="'.$this->field_name( $field ).'">';
				wp_editor( $content, $this->field_name( $field ), $settings );
				echo '</div>';
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Select field
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __select__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return '<!-- Options are empty in '.$field['id'].'! -->';
				$attrs = array();
				$width = !empty( $field['width'] ) && is_string( $field['width'] ) ? $field['width'] : '40%';
				$attrs['name'] = $this->field_name( $field );
				if( isset( $field['select2'] ) && $field['select2'] == false )
					$attrs['class'] = 'no-select2';
				$attrs['style'] = "width:{$width};";

				$select2 = array();

				if( !empty( $field['select2'] ) && is_array( $field['select2'] ) ){
					$select2 = $select2 + $field['select2'];
					if( !empty( $field['select2']['ajax'] ) && is_array( $field['select2']['ajax'] ) ){
						unset( $select2['ajax'] );
					}
				}

				$attrs['data-ipanel-select2'] = json_encode( $select2 );

				ob_start();
				?>
				<select <?php echo $this->_generate_attributes_string( $attrs, "'" ); ?>>
					<?php
						foreach( $field['options'] as $key => $option ){
							if( is_array( $option ) && !empty( $option[0] ) && is_array( $option[1] ) && !empty( $option[1] ) ){
								echo '<optgroup label="'.$option[0].'">';
								foreach( $option[1] as $sub_item_key => $sub_item_value )
									echo '<option value="'.$sub_item_key.'" '.selected( @$field['value'], $sub_item_key, false ).'>'.$sub_item_value.'</option>';
								echo '</optgroup>';
							} else if( is_string( $option ) )
								echo '<option value="'.$key.'" '.selected( @$field['value'], $key, false ).'>'.$option.'</option>';
						}
					?>
				</select>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Multi-Select field
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __multiselect__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return '<!-- Options are empty in '.$field['id'].'! -->';
				$width = !empty( $field['width'] ) && is_string( $field['width'] ) ? $field['width'] : '40%';
				$attrs = array();
				$sortable = !( isset( $field['sortable'] ) && !$field['sortable'] );
				if( isset( $field['select2'] ) && $field['select2'] == false )
					$attrs['class'] = 'no-select2';
				if( $sortable )
					$attrs['data-sortable'] = true;
				$attrs['style'] = "width:{$width};";
				$attrs['name'] = $this->field_name( $field );
				$attrs['multiple'] = 'multiple';
				$value = !empty( $field['value'] ) && is_array( $field['value'] ) ? $field['value'] : array();
				ob_start();
				?>
				<select <?php echo $this->_generate_attributes_string( $attrs ); ?>>
					<?php
						foreach( $field['options'] as $key => $option ){
							if( is_array( $option ) && !empty( $option[0] ) && is_array( $option[1] ) && !empty( $option[1] ) ){
								echo '<optgroup label="'.$option[0].'">';
								foreach( $option[1] as $sub_item_key => $sub_item_value )
									echo '<option value="'.$sub_item_key.'" '.( in_array( $sub_item_key, $value ) ? 'selected="selected"' : '' ).'>'.$sub_item_value.'</option>';
								echo '</optgroup>';
							} else if( is_string( $option ) )
								echo '<option value="'.$key.'" '.( in_array( $key, $value ) ? 'selected="selected"' : '' ).'>'.$option.'</option>';
						}
					?>
				</select>
				<?php
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Radio field
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __radio__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return '<!-- Options are empty in '.$field['id'].'! -->';
				$attrs = array();
				$attrs['name'] = $this->field_name( $field );
				$attrs['type'] = 'radio';
				ob_start();
				foreach( $field['options'] as $key => $value ) {
					$attrs['value'] = $key;
					if( isset( $field['value'] ) && $field['value'] == $key )
						$attrs['checked'] = 'checked';
					else
						if( isset( $attrs['checked'] ) )
							unset( $attrs['checked'] );
					?>
					<label class="ipanel-radio-option">
						<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
						<?php echo $value; ?>
					</label>
					<?php
				}
				return array( 'input' => ob_get_clean() );
			}



			/*
			 * Radio field
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __image_radio__( $field ) {
				if( empty( $field['options'] ) || !is_array( $field['options'] ) )
					return 'Something is wrong with "'.$field['id'].'" options.';
				ob_start();
				foreach( $field['options'] as $option_id => $option ) {

					if( empty( $option['label'] ) || !is_string( $option['image'] ) || empty( $option['label'] ) || !is_string( $option['label'] )  )
						continue;

					$attrs 			= array();
					$attrs['name']  = $this->field_name( $field );
					$attrs['type']  = 'radio';
					$attrs['value'] = $option_id;
					$checked 		= isset( $field['value'] ) && $option_id == $field['value'];

					if( $checked )
						$attrs['checked'] = 'checked';
					?>
					<label class="ipanel-image-radio-option <?php echo ( $checked ? 'checked' : 'unchecked' ); ?>">
						<input <?php echo $this->_generate_attributes_string( $attrs ); ?>/>
						<img src="<?php echo $option['image']; ?>" alt="<?php echo $option['label']; ?>" title="<?php echo $option['label']; ?>"/>
						<i></i>
					</label>
				<?php

				}
				return array( 'input' => ob_get_clean() );
			}



			/*
			 * Wordpress Media
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __media__( $field ) {

				$button_text = !empty( $field['button-text'] ) && is_string( $field['button-text'] ) ? $field['button-text'] : __( 'Open Media', $this->_panel['settings']['text_domain'] );
				$values = !empty( $field['value'] ) && is_array( $field['value'] ) ? $field['value'] : array();
				$sortable = !( isset( $field['sortable'] ) && !$field['sortable'] );
				$multiple = true;
				if( isset( $field['multiple'] ) && $field['multiple'] == false )
					$multiple = false;

				ob_start();
				?>
				<div data-url="{url}">
					<span class="remove">x</span>
					<input type="text" readonly value="{url}" name="{name}"/>
				</div>
				<?php
				$items_html = ob_get_clean();

				$index  = 0;
				ob_start();
				?>

				<div class="ipanel-media-field-wrapper">
					<div class="ipanel-media-items-container" <?php echo $sortable ? 'data-sortable' : ''; ?>>
						<?php foreach( $values as $value ) {
							echo str_replace(
								array(
									'{url}',
									'{name}'
								),
								array(
									$value,
									$this->field_name( $field )
								),
								$items_html
							);
						} ?>
					</div>
					<?php
						$attrs = array(
							'class' => 'ipanel-open-media',
							'data-name' => $this->field_name( $field ),
							'data-multiple' => (int) $multiple
						);
						if( !empty( $field['frame-title'] ) && is_string( $field['frame-title'] ) )
							$attrs['data-frame-title'] = $field['frame-title'];
						if( !empty( $field['frame-button-text'] ) && is_string( $field['frame-button-text'] ) )
							$attrs['data-frame-button-text'] = $field['frame-button-text'];
					?>
					<button <?php echo $this->_generate_attributes_string( $attrs ); ?>><?php echo $button_text; ?></button>
					<script type="text/html"><?php echo $items_html; ?></script>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );

			}



			/*
			 * Wordpress Media
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __ajax_uploader__( $field ) {

				ob_start();
				$label    = !empty( $field['button-label'] ) && is_string( $field['button-label'] ) ? $field['button-label'] : __( 'Choose File', $this->_panel['settings']['text_domain'] );
				$values   = !empty( $field['value'] ) && is_array( $field['value'] ) ? $field['value'] : array();
				$multiple = isset( $field['multiple'] ) && $field['multiple'];
				$sortable = !( isset( $field['sortable'] ) && !$field['sortable'] );
				$index    = 0;

				?>

				<div class="ipanel-ajax-uploader-wrapper">
					<div class="ipanel-ajax-uploader-container" data-__items_count__="<?php echo ( empty( $values ) ? 0 : count( $values ) ); ?>" <?php echo $sortable ? 'data-sortable' : ''; ?>>
						<?php foreach( $values as $val ) {
							++$index;
							if( empty( $val['url'] ) || !is_string( $val['url'] ) || empty( $val['file'] ) || !is_string( $val['file'] ) )
								continue;
						?>
						<div>
							<?php $field['__type__'] = 'url'; ?>
							<span class="remove">x</span>
							<input type="text" readonly value="<?php echo $val['url']; ?>" name="<?php echo str_replace( '[__index__]', '['.$index.']', $this->field_name( $field  ) ); ?>"/>
							<?php $field['__type__'] = 'file'; ?>
							<input type="hidden" readonly value="<?php echo $val['file']; ?>" name="<?php echo str_replace( '[__index__]', '['.$index.']', $this->field_name( $field  ) ); ?>"/>
						</div>
						<?php } ?>
					</div>
					<label class="ajax-uploader-button">
						<span><?php echo $label; ?></span>
						<?php

							$attrs = array(
								'type' => 'file',
								'class' => 'ipanel-ajax-uploader-file-input',
								'name' => 'file[]'
							);
							if( $multiple )
								$attrs['multiple'] = 'multiple';
						?>
						<input <?php echo $this->_generate_attributes_string($attrs); ?> data-file-name="<?php $field['__type__'] = 'file'; echo $this->field_name( $field ); ?>" data-url-name="<?php $field['__type__'] = 'url'; echo $this->field_name( $field ); ?>"/>
						<div class="progress-bar"></div>
					</label>

				</div>


				<?php

				return array( 'input' => ob_get_clean() );

			}


			/*
			 * Slider
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __slider__( $field ) {
				$val = 0;
				if( !empty( $field['value'] ) && is_numeric( $field['value'] ) )
					$val = $field['value'];
				$options = array(
					'animate' => true
				);
				if( !empty( $field['options'] ) && is_array( $field['options'] ) )
					$options = array_merge( $options, $field['options'] );

				$options['has-tooltip'] = 0;
				if( !empty( $field['tooltip'] ) && is_string( $field['tooltip'] ) )
					$options['has-tooltip'] = 1;

				ob_start();
				?>
				<div class="ipanel-slider-container">
					<script type="text/html">
						<?php if( $options['has-tooltip'] ){
							echo '<span class="ipanel-slider-tooltip">'.str_replace(
									'%s',
									"<span class='value'>{$val}</span>",
									$field["tooltip"]
								).'</span>';
						} ?>
					</script>
					<div class="ipanel-slider" data-options='<?php echo json_encode( $options ); ?>'></div>
					<input type="hidden" name="<?php echo $this->field_name( $field ); ?>" value="<?php echo $val; ?>" />
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}

			/*
			 * Date picker
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __date__( $field ) {
				ob_start();
				$value = !empty( $field['value'] ) && is_string( $field['value'] ) ? $field['value'] : '';
				$options = !empty( $field['options'] ) && is_array( $field['options'] ) ? $field['options'] : array();
				$attrs = array(
					'type' => 'text',
					'name' => $this->field_name( $field ),
					'class' => 'ipanel-date-picker',
					'value' => $value,
					'data-options' => json_encode( $options )
				);
				?>
				<div class="ipanel-date-picker-container">
					<input <?php echo $this->_generate_attributes_string( $attrs, "'" ); ?>/>
				</div>
				<?php
				return array( 'input' => ob_get_clean() );
			}


			/*
			 * Sorter
			 *
			 * @param (array) field array
			 *
			 * @since 2.0
			 * @access public
			 * @return string
			 */
			public function __sorter__( $field ) {
				if( empty( $field['groups'] ) || !is_array( $field['groups'] ) )
					return array( 'input' => $field['id'] . ' has not group array.' );
				$groups = $field['groups'];
				$has_value = false;
				if( !empty( $field['value'] ) && is_array( $field['value'] ) )
					$has_value = true;

				$groups = array_filter(
					$groups,
					create_function( '$item', 'return !(
							( empty( $item["label"] ) || !is_string( $item["label"] ) )
							|| ( empty( $item["items"] ) || !is_array( $item["items"] ) )
						);' )
				);
				if( empty( $groups ) )
					return array( 'input' => $field['id'] . ' has not any valid group!' );
				$col = floor( 12 / count( $groups ) );

				ob_start();
				?>
					<div class="ipanel-sortable-container">
				<?php
				foreach( $groups as $group_id => $group ) {
				?>
					<div class="ipanel-sorter-group <?php echo 'ipanel-col-'.$col; ?>" data-id="<?php echo $group_id; ?>">
						<h3><?php echo $group['label']; ?></h3>
						<div class="items-container">
							<?php
								if( $has_value ){
									$items = !empty( $field['value'][$group_id] ) ? $field['value'][$group_id] : array();
								} else {
									$items = $group['items'];
								}
								foreach( $items as $item_id => $item_value ) { ?>
									<div class="ipanel-sorter-item">
										<?php
											echo $item_value;
											$field['__group__'] = $group_id;
											$field['__item__'] = $item_id;
										?>
										<input type="hidden" name="<?php echo $this->field_name( $field ); ?>"  value="<?php echo $item_value; ?>" />
									</div>
								<?php }
							?>
						</div>

					</div>
				<?php
				}
				echo '</div>';
				echo '<div class="clear"></div>';
				return array( 'input' => ob_get_clean() );
			}




		}
	}